#pragma once

class MDVideoDataFormat;
class MDAudioDataFormat;

//=====================================================================================//
//                                 class MediaDetector                                 //
//=====================================================================================//
class MediaDetector
{
private:

	std::auto_ptr<MDVideoDataFormat> m_videoDataFormat;
	std::auto_ptr<MDAudioDataFormat> m_audioDataFormat;
	unsigned int m_duration;

public:
	MediaDetector(const std::string& file_name);
	~MediaDetector();
public:
	///	get video data format
	const MDVideoDataFormat* getVideoDataFormat() const;
	///	get audio data format
	const MDAudioDataFormat* getAudioDataFormat() const;
	///	get duration of data in milliseconds
	unsigned int getDuration() const;

};

//	get video data format
inline const MDVideoDataFormat* MediaDetector::getVideoDataFormat() const
{
	return m_videoDataFormat.get();
}
//	get audio data format
inline const MDAudioDataFormat* MediaDetector::getAudioDataFormat() const
{
	return m_audioDataFormat.get();
}
//	get duration of data in milliseconds
inline unsigned int MediaDetector::getDuration() const
{
	return m_duration;
}

//=====================================================================================//
//                               class MDVideoDataFormat                               //
//=====================================================================================//
class MDVideoDataFormat
{
private:

	unsigned int m_frameRate;
	unsigned int m_bitCount;
	unsigned int m_height;
	unsigned int m_width;

public:
	MDVideoDataFormat(const VIDEOINFOHEADER* vih,unsigned int frame_rate);
	~MDVideoDataFormat();
public:
	///	get width of bitmap in pixels
	unsigned int getWidth() const;
	///	get height of bitmap in pixels
	unsigned int getHeight() const;
	///	get the number of bits per pixel
	unsigned int getBitCount() const;
	///	get the frame rate, in frames per second.
	unsigned int getFrameRate() const;

};

//	get width of bitmap in pixels
inline unsigned int MDVideoDataFormat::getWidth() const
{
	return m_width;
}
//	get height of bitmap in pixels
inline unsigned int MDVideoDataFormat::getHeight() const
{
	return m_height;
}
//	get the number of bits per pixel
inline unsigned int MDVideoDataFormat::getBitCount() const
{
	return m_bitCount;
}
//	get the frame rate, in frames per second.
inline unsigned int MDVideoDataFormat::getFrameRate() const
{
	return m_frameRate;
}

//=====================================================================================//
//                               class MDAudioDataFormat                               //
//=====================================================================================//
class MDAudioDataFormat
{
private:

	unsigned int m_samplesPerSec;
	unsigned int m_bitsPerSample;
	unsigned int m_formatTag;
	unsigned int m_channels;

public:
	MDAudioDataFormat(const WAVEFORMATEX* vih);
	~MDAudioDataFormat();
public:
	///	get sample rate, in samples per second (hertz)
	unsigned int getSamplesPerSec() const;
	///	get bits per sample 
	unsigned int getBitsPerSample() const;
	///	get waveform-audio format type
	unsigned int getFormatType() const;
	///	get number of channels in the waveform-audio data
	unsigned int getChannels() const;

};

//	get sample rate, in samples per second (hertz)
inline unsigned int MDAudioDataFormat::getSamplesPerSec() const
{
	return m_samplesPerSec;
}
//	get bits per sample 
inline unsigned int MDAudioDataFormat::getBitsPerSample() const
{
	return m_bitsPerSample;
}
//	get waveform-audio format type
inline unsigned int MDAudioDataFormat::getFormatType() const
{
	return m_formatTag;
}
//	get number of channels in the waveform-audio data
inline unsigned int MDAudioDataFormat::getChannels() const
{
	return m_channels;
}
