#include "StdAfx.h"
#include "Bitmap.h"

//=====================================================================================//
//                                    class Bitmap                                     //
//=====================================================================================//
Bitmap::Bitmap(const std::string& file_name)
: m_height(0)
, m_width(0)
, m_bpp(0)
{
	std::ifstream stream(file_name.c_str(),std::ios::binary);

	if(stream)
	{
		BITMAPFILEHEADER bmfh = {0};
		BITMAPINFOHEADER bmih = {0};

		stream.read(reinterpret_cast<char*>(&bmfh),sizeof(bmfh));
		stream.read(reinterpret_cast<char*>(&bmih),sizeof(bmih));

		if(stream && (bmfh.bfType == 0x4d42))
		{
			m_height = bmih.biHeight;
			m_width = bmih.biWidth;
			m_bpp = bmih.biBitCount;

			const int to_image_bits = bmfh.bfOffBits - (sizeof(bmfh)+sizeof(bmih));
			const int image_size = (bmfh.bfSize - bmfh.bfOffBits);
			if((to_image_bits >= 0) && (image_size > 0))
			{
				stream.seekg(to_image_bits,std::ios::cur);
				m_data.resize(image_size);
				stream.read(&m_data[0],image_size);
			}
		}
	}
}

Bitmap::~Bitmap()
{
}
//	get bit per pixel
unsigned int Bitmap::getBitPerPixel() const
{
	return m_bpp;
}
//	get height in pixels
unsigned int Bitmap::getHeight() const
{
	return m_height;
}
//	get width in pixels
unsigned int Bitmap::getWidth() const
{
	return m_width;
}
//	get image data
const Bitmap::data_t& Bitmap::data() const
{
	return m_data;
}
