#include "StdAfx.h"
#include "FlvCompiler.h"

#include "Bitmap.h"

namespace
{

//=====================================================================================//
//                            void writeToStreamCallback()                             //
//=====================================================================================//
void CALLBACK writeToStreamCallback(LPVOID lpParam,LPCVOID lpData, DWORD nLen)
{
	std::ostream& stream = *reinterpret_cast<std::ostream*>(lpParam);
	stream.write(reinterpret_cast<const char*>(lpData),nLen);
	stream.flush();
}

}

//=====================================================================================//
//                                  class FlvCompiler                                  //
//=====================================================================================//
FlvCompiler::FlvCompiler(const std::string& file_name)
: m_file(file_name.c_str(),std::ios::binary)
, m_hScreenVideoEncoder(0)
, m_hFlvStream(0)
, m_height(0)
, m_width(0)
{
}

FlvCompiler::~FlvCompiler()
{
	if(m_hFlvStream)
		ftFlvStreamDestroy(m_hFlvStream);
	if(m_hScreenVideoEncoder)
		ftScreenVideoEncoderDestroy(m_hScreenVideoEncoder);
}
//	add frame to flv stream
bool FlvCompiler::addFrame(const std::string& image_file_name,unsigned int timestamp)
{
	Bitmap bitmap(image_file_name);

	if(!bitmap.data().empty() && (bitmap.getBitPerPixel() == 24))
	{
		if(checkFlvStream(bitmap.getWidth(),bitmap.getHeight()))
		{
			if(m_hScreenVideoEncoder)
			{
				const Bitmap::data_t& image = bitmap.data();
				ftScreenVideoEncodeFrame(m_hScreenVideoEncoder,&image[0],image.size());

				LPCVOID lpData = 0;
				DWORD cbBytes = 0;
				ftScreenVideoGetEncodedFrame(m_hScreenVideoEncoder,&lpData,&cbBytes);

				ftFlvStreamPutVideoData(m_hFlvStream,lpData,cbBytes,timestamp);

				return true;
			}
		}
	}

	return false;
}
//	check flv stream state and parameters
bool FlvCompiler::checkFlvStream(unsigned int width,unsigned int height)
{
	if(!m_hFlvStream)
	{
		//	create flv stream
		FTVIDEOINFO ftvi = {FT_VIDEOCODEC_SCREENVIDEO,width,height};
		FTFLVSTREAMINFO ftflvsi = {&ftvi,0,&writeToStreamCallback,&m_file};
		m_hFlvStream = ftFlvStreamCreate(&ftflvsi);

		//	create Screen Video encoder
		const unsigned int block_size = 16;
		m_hScreenVideoEncoder = ftScreenVideoEncoderCreate(width,height,block_size);

		m_width = width;
		m_height = height;
	}

	return m_hFlvStream && (m_width == width) && (m_height == height);
}
